open Term

let rec string_of_term = function 
  | TAdd (e1, e2) -> 
      Printf.sprintf "%s + %s"
         (string_of_complex_term e1) (string_of_complex_term e2)
  | TEq (e1, e2) -> 
      Printf.sprintf "%s = %s"
         (string_of_complex_term e1) (string_of_complex_term e2)
  | TLeq (e1, e2) -> 
      Printf.sprintf "%s < %s"
         (string_of_complex_term e1) (string_of_complex_term e2)
  | TSub (e1, e2) -> 
      Printf.sprintf "%s - %s"
         (string_of_complex_term e1) (string_of_complex_term e2)
  | TMul (e1, e2) -> 
      Printf.sprintf "%s * %s"
         (string_of_complex_term e1) (string_of_complex_term e2)
  | TLet (x, e1, e2) -> 
      Printf.sprintf "let %s = %s in %s" x
         (string_of_term e1) (string_of_term e2)
  | TLetTuple (x,y, e1, e2) -> 
      Printf.sprintf "let %s, %s = %s in %s" x y
         (string_of_term e1) (string_of_term e2)
  | TFun (x, e) -> 
      Printf.sprintf "fun %s -> %s" x (string_of_term e)
  | TApp (e1, e2) -> 
      Printf.sprintf "%s %s"
         (string_of_complex_term e1) (string_of_complex_term e2)
  | TPair (e1, e2) -> 
      Printf.sprintf "%s, %s"
         (string_of_term e1) (string_of_term e2)
  | TIf (e1, e2, e3) -> 
      Printf.sprintf "if %s then %s else %s"
         (string_of_term e1) (string_of_term e2) (string_of_term e3)
  | TFix (f, x, e) -> 
      Printf.sprintf "fix %s %s -> %s" f x (string_of_term e)
  | TRef t -> 
      Printf.sprintf "ref %s" (string_of_complex_term t)
  | TAss (t1, t2) -> 
      Printf.sprintf "%s := %s"
        (string_of_term t1) (string_of_term t2)
  | TSeq (t1, t2) -> 
      Printf.sprintf "%s;%s" (string_of_term t1) (string_of_term t2)

  | TVar _ | TCst _ | TDeref _ as e -> string_of_complex_term e

and string_of_complex_term = function 
  | TVar x -> x
  | TCst x -> string_of_value x
  | TDeref t -> 
      Printf.sprintf "!%s" (string_of_complex_term t)
  | e -> Printf.sprintf "(%s)" (string_of_term e)

and string_of_value = function 
  | VUnit -> "()"
  | VInt n -> string_of_int n
  | VBool b -> string_of_bool b
  | VFun _ -> "<fun>"
  | VRef r -> Printf.sprintf "ref %s" (string_of_value !r)
  | VPair (v1, v2) -> 
      Printf.sprintf "(%s, %s)"
         (string_of_value v1) (string_of_value v2)

let term_of_string s = 
  let lex = Lexing.from_string s in 
  try
    Parser.term Lexer.lexer lex
  with e ->
   flush stdout; 
   Printf.eprintf "Error : %s near '%s'.\n" (Printexc.to_string e) (Lexing.lexeme lex);
   flush stderr;
   failwith "term_of_string"


