open Term
open Conv


let examples = [
  "let x = 2 in x";
  "2 + (3 * 4)";
  "2 = 2";
  "if 2 = 2 then 3 else 3";
  "(fun x -> x) 42";
  "let rec f x = 2 in f 3";
  "let rec fact n = 
      if n = 0 then 1 else n * (fact (n - 1))
   in fact 5";
  "let compose f g x = f (g x) in 
   let id x = x in 
   compose id id";
  "let swap c = let x, y = c in (y, x) in swap (1, 2)";
  "let s x y z = x z (y z) in 
   let k x y = x in 
   let i = s k k in 
   i 3"; 
   "ref 0; ref 1";
   "let a = ref 0 in a := 1; !a";
   "let a = ref 1 in 
    let b = ref 2 in 
    let c = ref (!a + !b) in 
    b := !c + 1;
    !b + !c";
  "let a = ref 1 in 
   let b = ref 1 in 
   let rec fibo n =
     if n = 0 then ()
     else 
       let t = !b in 
       b := !a + !b;
       a := t;
       fibo (n - 1)
   in fibo 29; !a"; 
  "let or a b = if a then true else b in 
   let rec fibo n = 
      if or (n = 0) (n = 1) then 1 else (fibo (n - 1)) + (fibo (n - 2))
   in fibo 25";
  "let rec plus n m = 
     if n = 0 then m else (plus (n - 1) m) + 1
   in 
   let rec mult n m = 
     if n = 0 then 0 else plus (mult (n - 1) m) m
   in 
   let rec power n m = 
     if m = 0 then 1 else mult n (power n (m - 1))
   in 
   power 2 12";
  "let is_prime n = 
    (* ceci est un très mauvais test de primalité *)
    let rec aux1 a = 
      let rec aux2 b = 
        if b = n then false
        else if (a * b) = n then 
          true
        else 
          aux2 (b+1)
      in 
      if a = n then true else
      if aux2 a then
        false
      else
        aux1 (a+1)
    in 
    aux1 2
  in 
  let rec next_prime n = 
    if is_prime n then n else next_prime (n+1)
  in 
  next_prime 3000";
(* Oh !
  "let s x y z = x z (y z) in 
   let k x y = x in 
   let i = s k k in 
   s i i (s i i)"; *)
]


let _ = 
  let safe f = 
    try 
      let before = (Unix.times ()).Unix.tms_utime in 
      f (); 
      let after = (Unix.times ()).Unix.tms_utime in 
      Printf.printf "\t Used time : %f\n" (after -. before);
      flush stdout
    with e -> 
      Printf.printf "Error : uncatch exception '%s'.\n" (Printexc.to_string e); 
  in 
  let print_eval x = 
    let t = term_of_string x in 
      begin
       Printf.printf "--Evaluating '%s' :\n" (string_of_term t);
       safe (fun () -> Printf.printf "- %s\n" 
              (string_of_value (Eval.eval t)));
              Printf.printf "-- Unref :\n"; safe (fun () -> 
       let unref = Unref.unref t in 
       (* Décommentez cette ligne pour afficher le code transformé:
          Printf.printf "%s \n" (string_of_term unref); 
        *)
       Printf.printf "? %s\n" (string_of_value (Eval.eval unref)))
      end
  in
  List.iter print_eval  examples
