%{
  open Term
%}

%token LPAR RPAR LET IN EQUAL LEQ
%token <string> IDENT
%token <int> INT
%token <bool> BOOL
%token PLUS TIMES MINUS FIX UNIT
%token IF THEN ELSE FUN ARROW COMMA REC
%token REF BANG COLEQ SEQ 
%token EOF

%left ELSE IN ARROW COMMA
%left SEQ
%left PLUS TIMES MINUS COLEQ
%nonassoc LPAR IDENT INT BOOL UNIT REF
%nonassoc BANG

%start term
%type <Term.t> term
%%

constant : 
  | INT {
    VInt $1
  }
  | BOOL {
    VBool $1
  }
  | UNIT {
    VUnit
  }

simple_term:
  | BANG simple_term {
    TDeref $2
  }
  | IDENT {
    TVar $1
  }
  | constant {
    TCst $1 
  }
  | LPAR term RPAR {
    $2
  }

term: 
  | simple_term {
    $1
  }
  | REF term {
    TRef $2
  }
  | term simple_term {
    TApp ($1, $2)
  }
  | term SEQ term {
    TSeq ($1, $3)
  }
  | term COLEQ term {
    TAss ($1, $3)
  }
  | simple_term PLUS simple_term {
    TAdd ($1, $3)
  }
  | simple_term MINUS simple_term {
    TSub ($1, $3)
  }
  | simple_term TIMES simple_term {
    TMul ($1, $3)
  }
  | simple_term EQUAL simple_term {
    TEq ($1, $3)
  }
  | simple_term LEQ simple_term {
    TLeq ($1, $3)
  }

  | IF term THEN term ELSE term {
    TIf ($2, $4, $6)
  }
  | term COMMA term {
    TPair ($1, $3)
  }
  | LET IDENT COMMA IDENT EQUAL term IN term {
    TLetTuple ($2, $4, $6, $8)
  }
  | LET IDENT list_ident EQUAL term IN term {
    TLet ($2, 
      List.fold_left (fun acc y -> TFun (y, acc)) $5 $3, $7)
  }
  | LET REC IDENT IDENT list_ident EQUAL term IN term {
    TLet ($3, TFix ($3, $4, 
      List.fold_left (fun acc y -> TFun (y, acc)) $7 $5), $9)
  }
  | FUN IDENT ARROW term {
    TFun ($2, $4)
  }
  | FIX IDENT IDENT ARROW term {
    TFix ($2, $3, $5)
  }

list_ident : 
  |  { [] }
  | list_ident IDENT {$2::$1}

%%
