open Term

let context t =
  let hole = "__thisthehole__" in 
  let code = Conv.term_of_string
  (" 
   let empty = (true, 42) in
   let is_empty l = 
     let x, y = l in 
     x
   in 
   let head l = 
     let x, y = l in 
     let hd, tl = y in 
     hd 
   in 
   let tail l = 
     let x, y = l in 
     let hd, tl = y in 
     tl
   in 
   let add l x v = 
       (false, ((x,v), l))
   in
   let rec get l x = 
     let y, v = head l in 
     if x = y then v else 
     get (tail l) x
   in
   let max x y = 
     if x < y then y else x 
   in  
   let new l = 
     let rec aux k l = 
       if is_empty l then 
         k+1
       else
         let x, v = head l in 
         aux (max x k) (tail l)
     in 
     aux 0 l
   in \n "^hole)
  in 
  Eval.subst hole t code


let destruct t f = TFun ("?s", 
   TLetTuple ("?x", "?s", TApp (t, TVar "?s"),  
    f (TVar "?s") (TVar "?x")))

let destruct2 t1 t2 f = TFun ("?s", 
   TLetTuple ("?x", "?s", TApp (t1, TVar "?s"),  
   TLetTuple ("?y", "?s", TApp (t2, TVar "?s"), 
   f (TVar "?s") (TVar "?x") (TVar "?y"))))

let rec cps = function
    | TSeq (t1, t2) -> failwith "not implemented yet"
    | TDeref t -> failwith "not implemented yet"
    | TAss (t1, t2) -> failwith "not implemented yet"
    | TRef t -> failwith "not implemented yet"

    | TVar _ 
    | TCst _ as e -> 
         TFun ("?s", TPair (e, TVar "?s"))
    | TApp (t1, t2) ->
         destruct2 (cps t1) (cps t2) (fun s f a ->TApp (TApp (f, a), s))
    | TFun (x, t1) -> 
          TFun ("?s", TPair (TFun (x, cps t1), TVar "?s"))
    | TFix (f, x, t1) -> 
          TFun ("?s", TPair (TFix (f, x, cps t1), TVar "?s"))
    | TLet (x, e1, e2) -> TFun ("?s1", 
         TLetTuple (x, "?s2",  TApp (cps e1, TVar "?s1"), 
             TApp (cps e2, TVar "?s2")))
    | TLetTuple (x, y, e1, e2) -> 
        destruct (cps e1) (fun s c -> 
         TLetTuple (x, y, c, TApp (cps e2, s)))
    | TPair (e1, e2) -> 
        destruct2 (cps e1) (cps e2) (fun s x y -> TPair (TPair (x,y), s))
    | TAdd (t1, t2) -> 
        destruct2 (cps t1) (cps t2) (fun s x y -> TPair (TAdd (x,y), s)) 
    | TSub (t1, t2) -> 
        destruct2 (cps t1) (cps t2) (fun s x y -> TPair (TSub (x,y), s)) 
    | TMul (t1, t2) -> 
        destruct2 (cps t1) (cps t2) (fun s x y -> TPair (TMul (x,y), s)) 
    | TEq (t1, t2) -> 
        destruct2 (cps t1) (cps t2) (fun s x y -> TPair (TEq (x,y), s)) 
    | TLeq (t1, t2) -> 
        destruct2 (cps t1) (cps t2) (fun s x y -> TPair (TLeq (x,y), s)) 
    | TIf (t1, t2, t3) -> 
        destruct (cps t1) (fun s x -> 
          TIf (x, TApp (cps t2, s), TApp (cps t3, s)))
           
let unref t = 
   context (
     TLetTuple ("res", "state", 
     TApp (cps t, TVar "empty"), TVar "res"))

        

